#!/usr/local/bin/php -q 
<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Geeklog 1.3                                                               |
// +---------------------------------------------------------------------------+
// | lm.php                                                                    |
// |                                                                           |
// | Update a language file by merging it with english.php                     |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2004 by the following authors:                              |
// |                                                                           |
// | Author:  Dirk Haun         - dirk@haun-online.de                          |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
//
// $Id: lm.php,v 1.1.1.1 2004/02/05 14:16:33 dirk Exp $

$VERSION = '0.1';

// Prevent PHP from reporting uninitialized variables
error_reporting( E_ERROR | E_WARNING | E_PARSE | E_COMPILE_ERROR );

// name of the language file should be passed on the command line
$langfile = $GLOBALS['argv'][1];
if (empty ($langfile)) {
    echo "lm.php v{$VERSION}\n";
    echo "This is free software; see the source for copying conditions.\n\n";
    echo "Usage: {$GLOBALS['argv'][0]} langfile.php > new-langfile.php\n\n";
    exit;
}


// list of all variables accessed in the language file
$_DB_mysqldump_path         = '{$_DB_mysqldump_path}';
$_CONF['backup_path']       = '{$_CONF[\'backup_path\']}';
$_CONF['commentspeedlimit'] = '{$_CONF[\'commentspeedlimit\']}';
$_CONF['site_admin_url']    = '{$_CONF[\'site_admin_url\']}';
$_CONF['site_name']         = '{$_CONF[\'site_name\']}';
$_CONF['site_url']          = '{$_CONF[\'site_url\']}';
$_CONF['speedlimit']        = '{$_CONF[\'speedlimit\']}';
$_USER['username']          = '{$_USER[\'username\']}';

$failures                   = '{$failures}';
$from                       = '{$from}';
$fromemail                  = '{$fromemail}';
$qid                        = '{$qid}';
$shortmsg                   = '{$shortmsg}';
$successes                  = '{$successes}';
$topic                      = '{$topic}';
$type                       = '{$type}';

// load the English language file
require_once ('english.php');

// save the english text strings
$ENG01 = $LANG01;
$ENG02 = $LANG02;
$ENG03 = $LANG03;
$ENG04 = $LANG04;
$ENG05 = $LANG05;
$ENG06 = $LANG06;
$ENG07 = $LANG07;
$ENG08 = $LANG08;
$ENG09 = $LANG09;
$ENG10 = $LANG10;
$ENG11 = $LANG11;
$ENG12 = $LANG12;
// there are no $LANG13-$LANG19
$ENG20 = $LANG20;
$ENG21 = $LANG21;
$ENG22 = $LANG22;
$ENG23 = $LANG23;
$ENG24 = $LANG24;
$ENG25 = $LANG25;
// there is no $LANG26
$ENG27 = $LANG27;
$ENG28 = $LANG28;
$ENG29 = $LANG29;
$ENG30 = $LANG30;
$ENG31 = $LANG31;
$ENG32 = $LANG32;
$ENG33 = $LANG33;

$ENGMS = $MESSAGE;
$ENGAC = $LANG_ACCESS;
$ENGDB = $LANG_DB_BACKUP;
$ENGBT = $LANG_BUTTONS;
$ENG404 = $LANG_404;
$ENGLO = $LANG_LOGIN;

// now load the language file we want to update
require_once ($langfile);

function separator()
{
    echo "###############################################################################\n";
}

/**
* Merge two language arrays
*
* This function does all the work. Any missing text strings are copied
* over from english.php. Also does some pretty-printing.
*
*/
function mergeArrays($ENG, $OTHER, $arrayName, $comment = '')
{
    $numElements = sizeof($ENG);
    $counter = 0;

    separator();
    if (!empty ($comment)) {
        echo "# $comment\n";
    }
    echo "\n\${$arrayName} = array(\n";

    foreach ($ENG as $key => $txt) {
        $counter++;
        if (is_numeric ($key)) {
            echo "    $key => ";
        } else {
            echo "    '$key' => ";
        }
        $newtxt = '';
        if (empty ($OTHER[$key])) {
            // string does not exist in other language - use English text
            $newtxt = $txt;
        } else {
            // string exists in other language - keep it
            $newtxt = $OTHER[$key];
        }

        $newtxt = str_replace ("\n", '\n', $newtxt);

        if (strpos ($newtxt, '{$') === false) {
            // text does not contain variables, so enclose it in single quotes
            $newtxt = str_replace ("'", "\'", $newtxt);
            $quotedtext = "'" . $newtxt . "'";
        } else {
            // text contains variables
            $newtxt = str_replace ('$', '\$', $newtxt);
            $newtxt = str_replace ('{\$', '{$', $newtxt);
            $newtxt = str_replace ('"', '\"', $newtxt);
            $quotedtext = '"' . $newtxt . '"';
        }

        if ($counter != $numElements) {
            $quotedtext .= ',';
        }
        echo "$quotedtext\n";
    }

    echo ");\n\n";
}

/**
* Read the credits / copyright from the other language file.
* Assumes that it starts and ends with a separator line of # signs.
*
*/
function readCredits($langfile)
{
    $credits = array ();

    $firstcomment = false;

    $fh = fopen ($langfile, 'r');
    if ($fh !== false) {
        while (true) {
            $line = fgets ($fh);
            if ($firstcomment) {
                $credits[] = $line;
                if (strstr ($line, '#####') !== false) {
                    // end of credits reached
                    break;
                }
            } else {
                if (strstr ($line, '#####') !== false) {
                    // start of credits
                    $firstcomment = true;
                    $credits[] = $line;
                }
            }
        }
        fclose ($fh);
    }

    return ($credits);
}


// MAIN

$credits = readCredits ($langfile);

// output starts here ...

echo "<?php\n\n";

foreach ($credits as $c) {
    echo "$c"; // Note: linefeeds are part of the credits
}

echo "\n\$LANG_CHARSET = '$LANG_CHARSET';\n\n";

separator();
echo "# Array Format:\n";
echo "# \$LANGXX[YY]:  \$LANG - variable name\n";
echo "#               XX    - file id number\n";
echo "#               YY    - phrase id number\n";
separator();
echo "\n";
separator();
echo "# USER PHRASES - These are file phrases used in end user scripts\n";
separator();
echo "\n";

mergeArrays($ENG01, $LANG01, 'LANG01', 'lib-common.php');
mergeArrays($ENG02, $LANG02, 'LANG02', 'calendar.php');
mergeArrays($ENG03, $LANG03, 'LANG03', 'comment.php');
mergeArrays($ENG04, $LANG04, 'LANG04', 'users.php');
mergeArrays($ENG05, $LANG05, 'LANG05', 'index.php');
mergeArrays($ENG06, $LANG06, 'LANG06', 'links.php');
mergeArrays($ENG07, $LANG07, 'LANG07', 'pollbooth.php');
mergeArrays($ENG08, $LANG08, 'LANG08', 'profiles.php');
mergeArrays($ENG09, $LANG09, 'LANG09', 'search.php');
mergeArrays($ENG10, $LANG10, 'LANG10', 'stats.php');
mergeArrays($ENG11, $LANG11, 'LANG11', 'article.php');
mergeArrays($ENG12, $LANG12, 'LANG12', 'submit.php');

separator();
echo "# ADMIN PHRASES - These are file phrases used in admin scripts\n";
separator();
echo "\n";

mergeArrays($ENG20, $LANG20, 'LANG20', 'admin/auth.inc.php');
mergeArrays($ENG21, $LANG21, 'LANG21', 'admin/block.php');
mergeArrays($ENG22, $LANG22, 'LANG22', 'admin/event.php');
mergeArrays($ENG23, $LANG23, 'LANG23', 'admin/link.php');
mergeArrays($ENG24, $LANG24, 'LANG24', 'admin/story.php');
mergeArrays($ENG25, $LANG25, 'LANG25', 'admin/poll.php');
mergeArrays($ENG27, $LANG27, 'LANG27', 'admin/topic.php');
mergeArrays($ENG28, $LANG28, 'LANG28', 'admin/user.php');
mergeArrays($ENG29, $LANG29, 'LANG29', 'admin/moderation.php');
mergeArrays($ENG30, $LANG30, 'LANG30', 'calendar.php');
mergeArrays($ENG31, $LANG31, 'LANG31', 'admin/mail.php');
mergeArrays($ENG32, $LANG32, 'LANG32', 'admin/plugins.php');
mergeArrays($ENG33, $LANG33, 'LANG33', 'admin/syndication.php');

mergeArrays($ENGMS,  $MESSAGE, 'MESSAGE', 'confirmation and error messages');
mergeArrays($ENGAC,  $LANG_ACCESS, 'LANG_ACCESS');
mergeArrays($ENGDB,  $LANG_DB_BACKUP, 'LANG_DB_BACKUP', 'admin/database.php');
mergeArrays($ENGBT,  $LANG_BUTTONS, 'LANG_BUTTONS');
mergeArrays($ENG404, $LANG_404, 'LANG_404', '404.php');
mergeArrays($ENGLO,  $LANG_LOGIN, 'LANG_LOGIN');

echo "?>";

?>
